#! /usr/bin/python

import sys
import os
import os.path
import glob
import time
import string
import ConfigParser
import Julian


class Tar:
    """
    Reprsente une archive tar
    """
    def __init__(self, name, rep, dest, compress = 1):
        """
        rep : le rpertoire archiv
        dest : le rpertoire dans lequel est stock l'archive
        compress = 0 si nous reprsentons une archive non compresse
        """
        # Nom de l'archive (ex: etc)
        self.name = name
        # Rpertoire archiv (ex: /etc)
        self.rep = rep
        # Rpertoire destination (ex: /sauvegarde/etc)
        self.dest = dest
        t = Julian.now()
        self.year = t.year
        self.month = t.month
        self.day = t.day
        # Le numro de jour correspondant  notre archive
        self.jday = Julian.julianDay(self.year, self.month, self.day)
        # Le numro de jour d'aujourd'hui
        self.today = self.jday
        # Si nous sommes une archive compresse
        self.compress = compress
        
        # Si le rpertoire destination n'existe pas, le crer
        if not os.path.isdir(self.dest):
            os.mkdir(self.dest)
        # Remettre les droits sur le rpertoire
        os.chmod(self.dest, 0700)


    def from_file(self,file):
        """
        Nous construit  partir d'un nom de fichier absolu
        file =  sauvegarde/etc/nom.y-m-d.tar[.gz]
        """
        self.dest = os.path.dirname(file)
        basefile = os.path.basename(file)
        parts = string.split(basefile, '.')
        self.name = parts[0]
        date = string.split(parts[1], '-')
        self.year = date[0]
        self.month= date[1]
        self.day = date[2]
        self.jday = Julian.julianDay(self.year, self.month, self.day)
        # se termine par .gz ?
        self.compress = len(parts) > 3 


    def archive_file(self):
        """
        Le nom complet de notre archive.
        (ex : /sauvegarde/etc/etc.2002-4-1.tar.gz)
        """
        ext = 'tar'
        if self.compress:
            ext = ext + '.gz'
        archive = "%s/%s.%s-%s-%s.%s" % (self.dest,
                                         self.name,
                                         self.year,
                                         self.month,
                                         self.day,
                                         ext)
        return archive
            
    def make_tar(self):
        """
        Construit Notre fichier archiv et le place
        dans le rpertoire dest.
        """
        tar = "tar cf"
        if self.compress:
            tar = tar + "z"
        tar = tar + ' ' + self.archive_file() + ' ' + self.rep
        #print tar
        os.system(tar)

        

    def del_tar(self):
        """
        Supprime notre archive du rpertoire
        """
        os.remove(self.archive_file())


    def del_old(self, f, n):
        """
        Nous supprime lorsque nous ne sommes plus d'actualit.
        f : nombre de jours entre sauvegardes conserves.
        n : nombre de sauvegardes conserves.
        """
        j = long(self.jday)
        t = long(self.today)
        f = int(f)
        n = int(n)
        if  (j <> t) and (j <= t) and \
           ( (j  % f <> 0) or (t - n * f > j ) ):
            self.del_tar()




if __name__ == '__main__':
    """
    Usage : sauvegarde.py fich.conf
    fich.conf : un fichier de configuration des sauvegardes qui sert
    galement de fichier log. Sa syntaxe est : 
    [nom_archive]
    dir = (rep  archiver)
    dir_save = (rep destination o placer les tar)
    number = (nombre d'archives  conserver en plus de l'archive courante)
    modulo = (les jours d mod modulo sont conservs)
    compress = (1|0 s'il faut compresser l'archive)
    """ 
    f_config = sys.argv[1]
    
    log = open(f_config + '.log', 'a') 
    today = time.strftime(' %Y-%m-%d : %Hh-%M ')
    log.write('#' * 10  + today + '#' * 60 + '\n')
    config = ConfigParser.ConfigParser()
    config.read(f_config)
    for section in config.sections():
        name = section
        rep = config.get(section, 'dir')
        dest = config.get(section, 'dir_save')
        n = config.get(section, 'number')
        f = config.get(section, 'modulo')
        compress = config.get(section, 'compress')
        tar = Tar(name, rep, dest, compress)
        try:
            tar.make_tar()
            log.write('sauvegarde: %s \n' % section)
        except:
	    print "erreur de sauvegarde"
        ## Efface les vieilles archives.
        archives = glob.glob(dest + '/*.tar.*')
        for a in archives: 
            try:
                tar = Tar(name, rep, dest, compress) 
                tar.from_file(a) 
                tar.del_old(f, n) 
            except: 
		print "impossible de supprimer les archives anciennes"





